class Result<T> {
  Success<T> _success;
  Failure<String> _failure;

  Result._success(success) {
    _success = Success<T>(success);
  }

  Result._failure(failure) {
    _failure = Failure<String>(failure);
  }

  Result.success([T data]) : this._success(data);

  Result.failure([String message]) : this._failure(message);

  bool get isSuccess => _success != null;

  bool get isFailure => _failure != null;

  Success<T> get asSuccess => _success;

  void ifSuccess(Function(T data) body) {
    if (isSuccess) {
      body(_success.data);
    }
  }

  void ifFailure(Function(String message) body) {
    if (isFailure) {
      body(_failure.message);
    }
  }
}

class Success<T> {
  const Success(this.data);

  final T data;
}

class Failure<String> {
  const Failure(this.message);

  final String message;
}
